/**********************************************************************
* FILENAME
*   block.h
*
* DESCRIPTION
*   Provides elementary functions which are necessary to implement 
*   arbitrary signal flow graphs in a block processing environment
*
* Rev 1.1  Nov. 29th, 2001 		C. Feldbauer <feldbauer@tugraz.at>
* Rev 1.2  Oct. 05th, 2005 		C. Feldbauer <feldbauer@tugraz.at>
**********************************************************************/
#ifndef __BLOCK__
#define __BLOCK__


/* node (split a signal into two branches) */
void block_float_copy( float *dst, float *src, int blksz);
void block_short_copy( short *dst, short *src, int blksz);


/* add */
void block_float_add( float *dst, float *src, int blksz);
void block_short_add( short *dst, short *src, int blksz);


/* minus */
void block_float_sub( float *dst, float *src, int blksz);
void block_short_sub( short *dst, short *src, int blksz);
void block_float_c_is_a_minus_b( float *c, float *a, float *b, int blksz);
void block_short_c_is_a_minus_b( short *c, short *a, short *b, int blksz);


/* multiply 2 sequences (modulation, windowing, ...) */
void block_float_multiply( float *dst, float *src, int blksz);
void block_short_multiply_q15( short *dst, short *src, int blksz);
/* note: block_short_multiply_q15() right-shifts the result by 15 bits */
void block_float_cmplx_multiply( float *dst, float *src, int blksz);
/* note: blksz is the number of real-imag pairs here */
void block_short_cmplx_multiply_q15( short *dst, short *src, int blksz);
/* note: blksz is the number of real-imag pairs here */
/* note: block_short_multiply_q15() right-shifts the result by 15 bits */


/* multiply a sequence by a constant */
void block_float_multiply_const( float *sig, float constant, int blksz);
void block_short_multiply_const_q15( short *sig, short constant, int blksz);
/* note:  block_short_multiply_const_q15() right-shifts the result by 15 bits */


/* negate a sequence */
void block_float_negate( float *sig, int blksz);
void block_short_negate( short *sig, int blksz);


/* block data type conversions */
void block_float2short( float *floatbuf, short *shortbuf, int blksz);
void block_short2float( short *shortbuf, float *floatbuf, int blksz);


/* sampling rate decimation */
/* downsampling at [0, dec_fact, 2*dec_fact, 3*dec_fact, ...] */
void block_float_decimate( float *dst, float *src, int dec_fact, int blksz);
void block_short_decimate( short *dst, short *src, int dec_fact, int blksz);
/* works only if blksz is an integer multiple of the dec_fact */
/* note: blksz is the size of the src buffer */
/* note: size of the dst buffer is blksz/dec_fact */


/* sampling rate decimation */
/* downsampling at [off, off+dec_fact, off+2*dec_fact, off+3*dec_fact, ...] */
void block_float_decimate_off( float *dst, float *src, int dec_fact, int off, int blksz);
void block_short_decimate_off( short *dst, short *src, int dec_fact, int off, int blksz);
/* works only if blksz is an integer multiple of the dec_fact */
/* important: 0 <= off < dec_fact */
/* note: blksz is the size of the src buffer */
/* note: size of the dst buffer is blksz/dec_fact */


/* sampling rate expansion */
/* the first non-zero sample is at index 0 */
void block_float_interpolate( float *dst, float *src, int int_fact, int blksz);
void block_short_interpolate( short *dst, short *src, int int_fact, int blksz);
/* note: blksz is the size of the src buffer */
/* note: size of the dst buffer is blksz*int_fact */


/* sampling rate expansion */
/* the first non-zero sample is at index off */
void block_float_interpolate_off( float *dst, float *src, int int_fact, int off, int blksz);
void block_short_interpolate_off( short *dst, short *src, int int_fact, int off, int blksz);
/* important: 0 <= off < int_fact */
/* note: blksz is the size of the src buffer */
/* note: size of the dst buffer is blksz*int_fact */


#endif
