/**********************************************************************
* FILENAME
*   fir_float.c
* 
* DESCRIPTION
*   Generic floating-point FIR block filter with private delay line.
* The function 'fir_float' expects a (non-overlapping) buffer for input
*	and output samples.
* This function uses 'DSPF_sp_fir_gen' as the core function, which is available
* as a hand-optimized assembly routine.
* 
* V3.0,  Sep. 2005,  C. Feldbauer <feldbauer@tugraz.at>
**********************************************************************/

#include "fir_float.h"
#include <stdlib.h>

void DSPF_sp_fir_gen(const float *x, const float *h, float * restrict r, int nh, int nr);
// defined in DSPF_sp_fir_gen.c or in dsp67x.lib


void fir_float_init( struct_fir_float *data, float *coeffs, int num_coeff, int blksz)
{
  int i;

  data->coeffs = coeffs;  // coefficients
  data->num_coeff = num_coeff;  // # of FIR coefficients  
  data->blksz = blksz;    // size of output block
  data->tmpbufsz = blksz + num_coeff - 1;
  data->tmpbuf = (float *)malloc( data->tmpbufsz * sizeof( float));
  
  // the filter core expects the coefficients in reversed order,
  // let's reverse them here (use the newly created tmpbuf).
  // NOTE! When the coeff array is used by multiple fir_float objects, it is reversed multiple times!
  memcpy(data->tmpbuf, coeffs, num_coeff * sizeof( float));
  for( i=0; i<num_coeff; i++)
    coeffs[i] = data->tmpbuf[num_coeff-1-i];
  
  // set tmpbuf zero for first block
  memset( data->tmpbuf, 0 , data->tmpbufsz * sizeof( float));  
}



void fir_float( struct_fir_float *data, float *sigbuf)
{
  int j;	
  
  for( j=0; j<data->num_coeff-1; j++)   
    data->tmpbuf[j] = data->tmpbuf[data->blksz+j];    // copy old data
  
  for(    ; j<data->tmpbufsz; j++)  
    data->tmpbuf[j] = sigbuf[j-(data->num_coeff-1)];  // copy new data

  // should we use memcpy/memmove instead?
  
  
  // calculate convolution
  DSPF_sp_fir_gen( data->tmpbuf, data->coeffs, sigbuf, data->num_coeff, data->blksz);
}
